(function ($, Drupal, once) {
  Drupal.behaviors.filesAndFolders = {
    attach: function (context, settings) {
      var currentFolderId = settings.filesAndFolders.currentFolderId;

      // Folder navigation in the sidebar
      once('folder-click', '.folder-structure .folder-item', context).forEach(function(element) {
        $(element).on('click', function(e) {
          e.preventDefault();
          e.stopPropagation();
          var $this = $(this);
          var folderId = $this.data('folder-id');
          loadFolderContent(folderId);
        });
      });

      // Folder navigation in the content area
      $(context).on('click', '.content-area .folder-item', function(e) {
        e.preventDefault();
        var folderId = $(this).data('folder-id');
        loadFolderContent(folderId);
      });

      // Back link in the content area
      $(context).on('click', '.content-area .back-link', function(e) {
        e.preventDefault();
        var parentFolderId = $(this).data('folder-id');
        loadFolderContent(parentFolderId);
      });

      // Create subfolder
      once('create-subfolder', '#create-subfolder', context).forEach(function(element) {
        $(element).on('click', function() {
          createSubfolder();
        });
      });

      // Add files
      once('add-files', '#add-files', context).forEach(function(element) {
        $(element).on('click', function() {
          showFileUploadModal();
        });
      });

      // File actions (edit, delete, move)
      once('file-actions', '.file-list', context).forEach(function(element) {
        $(element).on('click', '.edit', function(e) {
          e.preventDefault();
          var itemId = $(this).closest('tr').data('item-id');
          editItem(itemId);
        });

        $(element).on('click', '.delete', function(e) {
          e.preventDefault();
          var itemId = $(this).closest('tr').data('item-id');
          if (confirm('Are you sure you want to delete this item?')) {
            deleteItem(itemId);
          }
        });

        $(element).on('click', '.move', function(e) {
          e.preventDefault();
          var itemId = $(this).closest('tr').data('item-id');
          var itemType = $(this).closest('tr').hasClass('folder-item') ? 'folder' : 'file';
          showMoveModal(itemId, itemType);
        });
      });

      // Content Visibility
      once('visibility-select', '#visibility-select', context).forEach(function(element) {
        $(element).on('change', function() {
          updateVisibility($(this).val());
        });
      });

      // Notifications
      once('subscribe-button', '#subscribe-button', context).forEach(function(element) {
        $(element).on('click', function() {
          subscribeToNotifications();
        });
      });

      once('show-details-button', '#show-details-button', context).forEach(function(element) {
        $(element).on('click', function() {
          showNotificationDetails();
        });
      });

      function loadFolderContent(folderId) {
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/load/' + folderId),
          method: 'GET',
          success: function(response) {
            $('.content-area .file-list').html(response.content);
            currentFolderId = folderId;
            updateActiveFolder(folderId);
            updateBackLink(folderId);
          },
          error: function() {
            console.error('Error loading folder content');
          }
        });
      }

      function updateActiveFolder(folderId) {
        $('.folder-structure .folder-item').removeClass('active');
        $('.folder-structure .folder-item[data-folder-id="' + folderId + '"]').addClass('active');
      }

      function updateBackLink(folderId) {
        var parentFolderId = null;
        var folder = $('.folder-structure .folder-item[data-folder-id="' + folderId + '"]');
        if (folder.length > 0) {
          parentFolderId = folder.closest('li').data('parent-folder-id');
        }
        if (parentFolderId) {
          $('.content-area .back-link').data('folder-id', parentFolderId).show();
        } else {
          $('.content-area .back-link').hide();
        }
      }

      function createSubfolder() {
        var modal = $('<div class="modal"><div class="modal-content"></div></div>');
        var content = modal.find('.modal-content');
        
        content.append('<h2>Create Subfolder</h2>');
        content.append('<input type="text" id="new-folder-name" placeholder="Folder Name">');
        
        var parentFolderSelect = $('<select id="parent-folder-select"></select>');
        content.append('<label for="parent-folder-select">Parent Folder:</label>');
        content.append(parentFolderSelect);
        
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/get-folder-structure'),
          method: 'GET',
          success: function(response) {
            if (response.structure) {
              populateParentFolderSelect(parentFolderSelect, response.structure);
            }
          },
          error: function() {
            alert('An error occurred while loading folder structure.');
          }
        });
        
        content.append('<button id="create-folder-button">Create</button>');
        content.append('<button id="close-modal-button">Cancel</button>');
        
        $('body').append(modal);
        modal.show();
        
        $('#create-folder-button').on('click', function() {
          var folderName = $('#new-folder-name').val();
          var parentFolderId = $('#parent-folder-select').val();
          if (folderName) {
            $.ajax({
              url: Drupal.url('files-and-folders/ajax/create-folder'),
              method: 'POST',
              data: {
                folder_name: folderName,
                parent_folder_id: parentFolderId
              },
              success: function(response) {
                if (response.status === 'success') {
                  loadFolderContent(currentFolderId);
                  updateFolderStructure();
                  modal.remove();
                } else {
                  alert('Error creating folder: ' + response.message);
                }
              },
              error: function() {
                alert('An error occurred while creating the folder.');
              }
            });
          }
        });
        
        $('#close-modal-button').on('click', function() {
          modal.remove();
        });
      }

      function populateParentFolderSelect(select, structure, level = 0) {
        $.each(structure, function(id, folder) {
          var option = $('<option></option>').val(id).text('  '.repeat(level) + folder.title);
          select.append(option);
          if (folder.children) {
            populateParentFolderSelect(select, folder.children, level + 1);
          }
        });
      }

      function updateFolderStructure() {
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/get-folder-structure'),
          method: 'GET',
          success: function(response) {
            if (response.structure) {
              $('.folder-structure').html(buildFolderStructureHtml(response.structure));
              attachFolderEvents();
            }
          },
          error: function() {
            console.error('Error updating folder structure');
          }
        });
      }

      function buildFolderStructureHtml(structure, level = 0) {
        var html = '<ul' + (level === 0 ? ' class="folder-tree"' : '') + '>';
        $.each(structure, function(id, folder) {
          html += '<li class="folder-item" data-folder-id="' + id + '">';
          html += '<span class="folder-title">' + folder.title + '</span>';
          if (folder.children && Object.keys(folder.children).length > 0) {
            html += buildFolderStructureHtml(folder.children, level + 1);
          }
          html += '</li>';
        });
        html += '</ul>';
        return html;
      }

      function attachFolderEvents() {
        $('.folder-structure .folder-item').off('click').on('click', function(e) {
          e.preventDefault();
          e.stopPropagation();
          var folderId = $(this).data('folder-id');
          loadFolderContent(folderId);
        });
      }

      function showFileUploadModal() {
        var modal = $('<div class="modal"><div class="modal-content"></div></div>');
        var content = modal.find('.modal-content');
        
        content.append('<h2>Upload Files</h2>');
        content.append('<div id="drop-area"><p>Drag and drop files here or click to select files</p><input type="file" id="file-upload-input" multiple style="display: none;"></div>');
        content.append('<button id="upload-files-button">Upload</button>');
        content.append('<button id="close-modal-button">Cancel</button>');
        
        $('body').append(modal);
        modal.show();

        var dropArea = $('#drop-area');
        var fileInput = $('#file-upload-input');

        ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
          dropArea.on(eventName, preventDefaults);
        });

        ['dragenter', 'dragover'].forEach(eventName => {
          dropArea.on(eventName, highlight);
        });

        ['dragleave', 'drop'].forEach(eventName => {
          dropArea.on(eventName, unhighlight);
        });

        dropArea.on('drop', handleDrop);

        dropArea.on('click', function() {
          fileInput.click();
        });

        fileInput.on('change', function(e) {
          handleFiles(e.target.files);
        });

        $('#upload-files-button').on('click', function() {
          var files = $('#file-upload-input')[0].files;
          if (files.length > 0) {
            uploadFiles(files);
            modal.remove();
          } else {
            alert('Please select files to upload.');
          }
        });

        $('#close-modal-button').on('click', function() {
          modal.remove();
        });
      }

      function preventDefaults(e) {
        e.preventDefault();
        e.stopPropagation();
      }

      function highlight(e) {
        $('#drop-area').addClass('highlight');
      }

      function unhighlight(e) {
        $('#drop-area').removeClass('highlight');
      }

      function handleDrop(e) {
        var dt = e.originalEvent.dataTransfer;
        var files = dt.files;
        handleFiles(files);
      }

      function handleFiles(files) {
        var fileList = $('#drop-area').find('p');
        fileList.empty();
        fileList.append('Selected files:<br>');
        for (var i = 0; i < files.length; i++) {
          fileList.append(files[i].name + '<br>');
        }
        $('#file-upload-input')[0].files = files;
      }

      function uploadFiles(files) {
        var formData = new FormData();
        $.each(files, function(i, file) {
          formData.append('files[]', file);
        });
        formData.append('parent_folder_id', currentFolderId);

        $.ajax({
          url: Drupal.url('files-and-folders/ajax/add-files'),
          type: 'POST',
          data: formData,
          processData: false,
          contentType: false,
          success: function(response) {
            if (response.status === 'success') {
              loadFolderContent(currentFolderId);
              alert('Files uploaded successfully');
            } else {
              alert('Error uploading files: ' + response.message);
            }
          },
          error: function(xhr, status, error) {
            console.error('AJAX error:', status, error);
            alert('An error occurred while uploading files.');
          }
        });
      }

      function editItem(itemId) {
        var newName = prompt("Enter new name:");
        if (newName) {
          $.ajax({
            url: Drupal.url('files-and-folders/ajax/edit-item/' + itemId),
            method: 'POST',
            data: {
              new_name: newName
            },
            success: function(response) {
              if (response.status === 'success') {
                loadFolderContent(currentFolderId);
              } else {
                alert('Error editing item: ' + response.message);
              }
            },
            error: function() {
              alert('An error occurred while editing the item.');
            }
          });
        }
      }

      function deleteItem(itemId) {
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/delete-item/' + itemId),
          method: 'POST',
          success: function(response) {
            if (response.status === 'success') {
              loadFolderContent(currentFolderId);
            } else {
              alert('Error deleting item: ' + response.message);
            }
          },
          error: function() {
            alert('An error occurred while deleting the item.');
          }
        });
      }

      function updateVisibility(visibility) {
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/update-visibility'),
          method: 'POST',
          data: {
            folder_id: currentFolderId,
            visibility: visibility
          },
          success: function(response) {
            if (response.status === 'success') {
              // Update UI to reflect new visibility
            } else {
              alert('Error updating visibility: ' + response.message);
            }
          },
          error: function() {
            alert('An error occurred while updating visibility.');
          }
        });
      }

      function subscribeToNotifications() {
        // Implement subscription logic
        console.log('Subscribe to notifications');
      }

      function showNotificationDetails() {
        // Implement show details logic
        console.log('Show notification details');
      }

      function showMoveModal(itemId, itemType) {
        var modal = $('<div class="modal"><div class="modal-content"></div></div>');
        var content = modal.find('.modal-content');
        
        content.append('<h2>Move ' + itemType + '</h2>');
        var folderSelect = $('<select id="move-destination"></select>');
        content.append(folderSelect);
        
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/get-folder-structure'),
          method: 'GET',
          success: function(response) {
            if (response.structure) {
              populateFolderSelect(folderSelect, response.structure);
            }
          },
          error: function() {
            alert('An error occurred while loading folder structure.');
          }
        });
        
        content.append('<button id="move-item-button">Move</button>');
        content.append('<button id="close-modal-button">Cancel</button>');
        
        $('body').append(modal);
        modal.show();
        
        $('#move-item-button').on('click', function() {
          var newParentFolderId = $('#move-destination').val();
          moveItem(itemId, newParentFolderId);
          modal.remove();
        });
        
        $('#close-modal-button').on('click', function() {
          modal.remove();
        });
      }

      function populateFolderSelect(select, structure, level = 0) {
        $.each(structure, function(id, folder) {
          var option = $('<option></option>').val(id).text('  '.repeat(level) + folder.title);
          select.append(option);
          if (folder.children) {
            populateFolderSelect(select, folder.children, level + 1);
          }
        });
      }

      function moveItem(itemId, newParentFolderId) {
        $.ajax({
          url: Drupal.url('files-and-folders/ajax/move-item'),
          method: 'POST',
          data: {
            item_id: itemId,
            new_parent_folder_id: newParentFolderId
          },
          success: function(response) {
            if (response.status === 'success') {
              loadFolderContent(currentFolderId);
            } else {
              alert('Error moving item: ' + response.message);
            }
          },
          error: function() {
            alert('An error occurred while moving the item.');
          }
        });
      }
    }
  };
})(jQuery, Drupal, once);