<?php

namespace Drupal\files_and_folders\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

class FilesAndFoldersController extends ControllerBase {

  protected $filesAndFoldersManager;

  public function __construct($filesAndFoldersManager) {
    $this->filesAndFoldersManager = $filesAndFoldersManager;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('files_and_folders.manager')
    );
  }

  public function content(NodeInterface $node = null) {
    $accessChecker = \Drupal::service('files_and_folders.access_checker');
    $access = $accessChecker->access($node);
    if (!$access->isAllowed()) {
      throw new AccessDeniedHttpException();
    }

    try {
      $content = $this->loadFolderContent($node ? $node->id() : 'public');
      $folderStructure = $this->getFolderStructure();
      return [
        '#theme' => 'files_and_folders',
        '#folder' => $node ?? 'public',
        '#content' => $content,
        '#folder_structure' => $folderStructure,
        '#attached' => [
          'library' => ['files_and_folders/files-and-folders'],
          'drupalSettings' => [
            'filesAndFolders' => [
              'currentFolderId' => $node ? $node->id() : 'public',
            ],
          ],
        ],
      ];
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in content method: @message', ['@message' => $e->getMessage()]);
      return [
        '#markup' => $this->t('An error occurred while loading the folder contents.'),
      ];
    }
  }

  public function loadFolderContent($folderId = null) {
    try {
      \Drupal::logger('files_and_folders')->debug('Loading folder content for folderId: @folderId', ['@folderId' => $folderId]);
      $query = $this->entityTypeManager()->getStorage('node')->getQuery()
        ->condition('type', ['folder', 'file'], 'IN')
        ->sort('created', 'DESC')
        ->accessCheck(TRUE);

      if ($folderId && $folderId !== 'public') {
        $query->condition('field_parent_folder', $folderId);
      } else {
        $query->condition('field_parent_folder', NULL, 'IS NULL');
      }

      $nids = $query->execute();
      $nodes = $this->entityTypeManager()->getStorage('node')->loadMultiple($nids);
      $content = [];
      foreach ($nodes as $node) {
        $content[] = [
          'id' => $node->id(),
          'type' => $node->bundle(),
          'title' => $node->label(),
          'changed' => $node->getChangedTime(),
          'file_size' => $node->hasField('field_file') ? $node->get('field_file')->entity->getSize() : null,
          'file_url' => $node->hasField('field_file') ? $node->get('field_file')->entity->createFileUrl() : null,
        ];
      }

      $parentFolderId = $this->getParentFolderId($folderId);
      \Drupal::logger('files_and_folders')->debug('Returned content: @content', ['@content' => print_r($content, TRUE)]);
      return [
        'content' => $content,
        'parent_folder_id' => $parentFolderId,
      ];
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in loadFolderContent: @message', ['@message' => $e->getMessage()]);
      return [];
    }
  }

  private function getParentFolderId($folderId) {
    if ($folderId && $folderId !== 'public') {
      $folder = $this->entityTypeManager()->getStorage('node')->load($folderId);
      if ($folder) {
        return $folder->get('field_parent_folder')->target_id;
      }
    }
    return null;
  }

  public function getFolderStructure($parentId = null) {
    try {
      $query = $this->entityTypeManager()->getStorage('node')->getQuery()
        ->condition('type', 'folder')
        ->sort('title', 'ASC')
        ->accessCheck(TRUE);

      if ($parentId) {
        $query->condition('field_parent_folder', $parentId);
      } else {
        $query->condition('field_parent_folder', NULL, 'IS NULL');
      }

      $nids = $query->execute();
      $nodes = $this->entityTypeManager()->getStorage('node')->loadMultiple($nids);
      $structure = [];
      foreach ($nodes as $node) {
        $item = [
          'id' => $node->id(),
          'title' => $node->label(),
          'children' => $this->getFolderStructure($node->id()),
        ];
        $structure[$node->id()] = $item;
      }

      return $structure;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in getFolderStructure: @message', ['@message' => $e->getMessage()]);
      return [];
    }
  }

  public function ajaxLoadContent($folderId = null) {
    try {
      $folderContent = $this->loadFolderContent($folderId);
      return new JsonResponse([
        'content' => $this->renderFolderContents($folderContent['content']),
        'folderId' => $folderId,
        'parentFolderId' => $folderContent['parent_folder_id'],
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxLoadContent: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while loading the folder contents.'], 500);
    }
  }

  public function ajaxGetFolderStructure($folderId) {
    try {
      $structure = $this->getFolderStructure($folderId);
      return new JsonResponse(['structure' => $structure]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxGetFolderStructure: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while fetching the folder structure.'], 500);
    }
  }

  private function renderFolderContents($content) {
    $build = [
      '#theme' => 'files_and_folders_content',
      '#content' => $content,
    ];
    return \Drupal::service('renderer')->render($build);
  }

  public function ajaxCreateFolder(Request $request) {
    $folderName = $request->request->get('folder_name');
    $parentFolderId = $request->request->get('parent_folder_id');
    $result = $this->filesAndFoldersManager->createFolder($folderName, $parentFolderId);
    if ($result) {
      return new JsonResponse(['status' => 'success', 'message' => 'Folder created successfully']);
    } else {
      return new JsonResponse(['status' => 'error', 'message' => 'Failed to create folder'], 500);
    }
  }

  public function ajaxAddFiles(Request $request) {
    try {
      $parentFolderId = $request->request->get('parent_folder_id');
      $files = $request->files->get('files');
      $addedFiles = $this->filesAndFoldersManager->addFiles($files, $parentFolderId);
      return new JsonResponse([
        'status' => 'success',
        'files' => $addedFiles,
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxAddFiles: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while adding files.'], 500);
    }
  }

  public function ajaxUpdateVisibility(Request $request) {
    try {
      $folderId = $request->request->get('folder_id');
      $visibility = $request->request->get('visibility');
      $result = $this->filesAndFoldersManager->updateVisibility($folderId, $visibility);
      return new JsonResponse([
        'status' => $result ? 'success' : 'error',
        'message' => $result ? 'Visibility updated successfully' : 'Failed to update visibility',
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxUpdateVisibility: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while updating visibility.'], 500);
    }
  }

  public function ajaxEditItem(Request $request, $itemId) {
    try {
      $newName = $request->request->get('new_name');
      $result = $this->filesAndFoldersManager->editItem($itemId, $newName);
      return new JsonResponse([
        'status' => $result ? 'success' : 'error',
        'message' => $result ? 'Item updated successfully' : 'Failed to update item',
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxEditItem: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while editing the item.'], 500);
    }
  }

  public function ajaxDeleteItem($itemId) {
    try {
      $result = $this->filesAndFoldersManager->deleteItem($itemId);
      return new JsonResponse([
        'status' => $result ? 'success' : 'error',
        'message' => $result ? 'Item deleted successfully' : 'Failed to delete item',
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxDeleteItem: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while deleting the item.'], 500);
    }
  }

  public function ajaxSubscribe(Request $request) {
    try {
      $folderId = $request->request->get('folder_id');
      $groups = $request->request->get('groups');
      $teams = $request->request->get('teams');
      $users = $request->request->get('users');
      $noNotifications = $request->request->get('no_notifications');
      $result = $this->filesAndFoldersManager->subscribeToFolder($folderId, $groups, $teams, $users, $noNotifications);
      return new JsonResponse([
        'status' => $result ? 'success' : 'error',
        'message' => $result ? 'Subscription successful' : 'Failed to subscribe',
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxSubscribe: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while subscribing.'], 500);
    }
  }

  public function ajaxShowDetails($folderId) {
    try {
      $details = $this->filesAndFoldersManager->getFolderDetails($folderId);
      return new JsonResponse([
        'status' => 'success',
        'details' => $details,
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxShowDetails: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while fetching folder details.'], 500);
    }
  }

  public function ajaxMoveItem(Request $request) {
    try {
      $itemId = $request->request->get('item_id');
      $newParentFolderId = $request->request->get('new_parent_folder_id');
      $result = $this->filesAndFoldersManager->moveItem($itemId, $newParentFolderId);
      return new JsonResponse([
        'status' => $result ? 'success' : 'error',
        'message' => $result ? 'Item moved successfully' : 'Failed to move item',
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error in ajaxMoveItem: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while moving the item.'], 500);
    }
  }

  public function getFolderTitle($folderId) {
    $node = $this->entityTypeManager()->getStorage('node')->load($folderId);
    return $node ? $node->label() : $this->t('Files and Folders');
  }
}