<?php

namespace Drupal\files_and_folders\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Archiver\ArchiverManager;

class FilesAndFoldersManager {

  protected $entityTypeManager;
  protected $fileSystem;
  protected $fileRepository;
  protected $currentUser;
  protected $archiverManager;

  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    FileRepositoryInterface $file_repository,
    AccountProxyInterface $current_user,
    ArchiverManager $archiver_manager
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->fileRepository = $file_repository;
    $this->currentUser = $current_user;
    $this->archiverManager = $archiver_manager;
  }

  public function createFolder($folderName, $parentFolderId) {
    try {
      $folder = $this->entityTypeManager->getStorage('node')->create([
        'type' => 'folder',
        'title' => $folderName,
        'field_parent_folder' => $parentFolderId ? ['target_id' => $parentFolderId] : NULL,
        'uid' => $this->currentUser->id(),
      ]);
      $folder->save();
      return $folder->id();
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error creating folder: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  public function addFiles($files, $parentFolderId) {
    $addedFiles = [];
    foreach ($files as $file) {
      try {
        $contents = file_get_contents($file->getRealPath());
        $destination = 'public://' . $file->getClientOriginalName();
        $fileEntity = $this->fileRepository->writeData($contents, $destination, FileSystemInterface::EXISTS_RENAME);
        
        $node = $this->entityTypeManager->getStorage('node')->create([
          'type' => 'file',
          'title' => $file->getClientOriginalName(),
          'field_file' => [
            'target_id' => $fileEntity->id(),
            'alt' => $file->getClientOriginalName(),
            'title' => $file->getClientOriginalName(),
          ],
          'field_parent_folder' => $parentFolderId ? ['target_id' => $parentFolderId] : NULL,
          'uid' => $this->currentUser->id(),
        ]);
        $node->save();
        $addedFiles[] = [
          'id' => $node->id(),
          'name' => $node->label(),
          'size' => $fileEntity->getSize(),
          'mime' => $fileEntity->getMimeType(),
        ];
      } catch (\Exception $e) {
        \Drupal::logger('files_and_folders')->error('Error adding file: @message', ['@message' => $e->getMessage()]);
      }
    }
    return $addedFiles;
  }

  public function createZipFromFolder($folderId) {
    $folder = $this->entityTypeManager->getStorage('node')->load($folderId);
    if (!$folder || $folder->bundle() !== 'folder') {
      throw new \Exception('Invalid folder ID');
    }

    $tempDir = $this->fileSystem->getTempDirectory();
    $zipFileName = $folder->label() . '.zip';
    $zipFilePath = $tempDir . '/' . $zipFileName;

    $archiver = $this->archiverManager->getInstance(['filepath' => $zipFilePath]);
    $this->addFolderContentsToArchive($folder, $archiver);

    return $zipFilePath;
  }

  private function addFolderContentsToArchive($folder, $archiver, $basePath = '') {
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', ['folder', 'file'], 'IN')
      ->condition('field_parent_folder', $folder->id())
      ->accessCheck(TRUE);
    $nids = $query->execute();
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

    foreach ($nodes as $node) {
      if ($node->bundle() === 'folder') {
        $this->addFolderContentsToArchive($node, $archiver, $basePath . $node->label() . '/');
      } else {
        $file = $node->get('field_file')->entity;
        if ($file) {
          $archiver->add($file->getFileUri(), $basePath . $file->getFilename());
        }
      }
    }
  }

  public function updateVisibility($folderId, $visibility) {
    try {
      $folder = $this->entityTypeManager->getStorage('node')->load($folderId);
      if (!$folder || $folder->bundle() !== 'folder') {
        throw new \Exception('Invalid folder ID');
      }
      $folder->set('field_visibility', $visibility);
      $folder->save();
      return TRUE;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error updating visibility: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  public function editItem($itemId, $newName) {
    try {
      $item = $this->entityTypeManager->getStorage('node')->load($itemId);
      if (!$item) {
        throw new \Exception('Invalid item ID');
      }
      $item->setTitle($newName);
      $item->save();
      return TRUE;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error editing item: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  public function deleteItem($itemId) {
    try {
      $item = $this->entityTypeManager->getStorage('node')->load($itemId);
      if (!$item) {
        throw new \Exception('Invalid item ID');
      }
      $item->delete();
      return TRUE;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error deleting item: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  public function subscribeToFolder($folderId, $groups, $teams, $users, $noNotifications) {
    // Implement subscription logic here
    // This is a placeholder and should be implemented based on your specific requirements
    return TRUE;
  }

  public function getFolderDetails($folderId) {
    try {
      $folder = $this->entityTypeManager->getStorage('node')->load($folderId);
      if (!$folder || $folder->bundle() !== 'folder') {
        throw new \Exception('Invalid folder ID');
      }
      
      $details = [
        'id' => $folder->id(),
        'name' => $folder->label(),
        'created' => $folder->getCreatedTime(),
        'changed' => $folder->getChangedTime(),
        'owner' => $folder->getOwner()->getDisplayName(),
        'visibility' => $folder->get('field_visibility')->value,
        // Add more details as needed
      ];
      
      return $details;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error getting folder details: @message', ['@message' => $e->getMessage()]);
      return NULL;
    }
  }

  public function moveItem($itemId, $newParentFolderId) {
    try {
      $item = $this->entityTypeManager->getStorage('node')->load($itemId);
      if (!$item) {
        throw new \Exception('Invalid item ID');
      }
      $item->set('field_parent_folder', $newParentFolderId);
      $item->save();
      return TRUE;
    } catch (\Exception $e) {
      \Drupal::logger('files_and_folders')->error('Error moving item: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }
}